package com.jdbernard.wdiwtlt

import com.zaxxer.hikari.HikariConfig

public class ConfigWrapper {

    public static final String LIBRARY_DIR_KEY = "library.dir"
    public static final String DB_CONFIG_FILE_KEY = "database.config.file"

    Properties configProps
    Map<String, String> env = System.getenv()

    public ConfigWrapper() {
        if (env.WDIWTLT_CONFIG_FILE) {
            if (tryLoadConfigFile(new File(env.WDIWTLT_CONFIG_FILE))) {
                return } }

        if (tryLoadConfigFile(new File(env.HOME, ".wdiwtlt.properties"))) return

        try {
            ConfigWrapper.getResourceAsStream("/com/jdbernard/wdiwtlt/db/default.properties")
                .withStream { tryLoadConfig(is) } }
        catch (Exception e) {} }

    public ConfigWrapper(File propertiesFile) {
        this.configPropertiesFile = configPropertiesFile }

    public String getLibraryRootPath() {
        // 1. Check config properties
        if (configProps && configProps[LIBRARY_DIR_KEY])
            return configProps[LIBRARY_DIR_KEY]

        // 2. Check environment variable 
        if (env.WDIWTLT_LIBRARY_DIR)
            return env.WDIWTLT_LIBRARY_DIR

        return null }

    public String getHikariConfig() {
        Properties props = new Properties()

        // 1. Check config properties for database config file
        if (configProps) {

            // 1.1 Look for a reference to a dedicated config file
            if (configProps[DB_CONFIG_FILE_KEY]) {
                File cf = new File(configProps[DB_CONFIG_FILE_KEY])
                if (cf.exists() && cf.isFile())
                    cf.withInputStream { props.load(it) } }

            // 1.2 Look for prefixed properties in this config file
            else if (configProps["database.config.dataSourceClassName"]) {
                props.putAll(configProps
                    .findAll { it.key.startsWith("database.config.") }
                    .collectEntries { [it.key[16..-1], it.value] } ) } }

        // 2. Look for environment variables
        if (!props) {
            if (env.WDIWTLT_DATASOURCE_CLASSNAME)
                props["dataSourceClassName"] = env.WDIWTLT_DATABASE_DATASOURCECLASSNAME

            if (env.WDIWTLT_DATASOURCE_PROPERTIES) {
                props.putAll(env.WDIWTLT_DATASOURCE_PROPERTIES.split(":")
                    .collect { it.split("=") }
                    .collectEntries { ["dataSource.${it[0]}", it[1]] }) } }

        // If properties exist, create and return HikariConfig
        if (props) return new HikariConfig(props)
        else return null }

    private boolean tryLoadConfigFile(File configFile) {
        if (!configFile.exists() || !configFile.isFile()) return false

        return configFile.withInputStream { tryLoadConfig(is) } }

    private boolean tryLoadConfig(InputStream is) {
        try {
            Properties props = new Properties()
            props.load(is)
            this.configProps = props }
        catch (Exception e) { return false }

        return true }
}
